@@ -4,8 +4,10 @@
 #' @param formula An object of class \code{formula}: a sympbolic 
 #' description of the model to be fitted. The details of model 
 #' specification are given under 'Details'.
-#'@param data A data frame or matrix containing the model response variable
-#' and covariates required by the \code{formula}.
+#' @param data An optional data frame, matrix or list required by 
+#' the formula. If not found in data, the variables are taken from 
+#' \code{environment(formula)}, typically the environment from which
+#'  \code{localtest} is called.
 #' @param na.action A function which indicates what should happen when the 
 #' data contain 'NA's. The default is 'na.omit'.
 #' @param der Number which determines any inference process. 
@@ -55,6 +57,7 @@
 #'@param ncores An integer value specifying the number of cores to be used
 #' in the parallelized procedure. If \code{NULL} (default), the number of cores 
 #' to be used is equal to the number of cores of the machine - 1.
+#'@param ci.level Level of bootstrap confidence interval. Defaults to 0.95 (corresponding to 95\%). Note that the function accepts a vector of levels.
 #' @param \ldots Other options.
 #' 
 #' 
@@ -124,7 +127,6 @@
 #' # localtest(height ~ s(age, by = sex), data = children, seed = 130853, 
 #' # der = 1, smooth = "splines") 
 #' 
-#' @useDynLib npregfast localtest_
 #' @importFrom stats na.omit runif
 #' @importFrom mgcv interpret.gam gam predict.gam
 #' @importFrom sfsmisc D1D2
@@ -141,7 +143,7 @@ localtest <- function(formula, data = data, na.action = "na.omit",
                       nboot = 500, h0 = -1.0, h = -1.0, nh = 30, 
                       kernel = "epanech", p = 3, kbin = 100, rankl = NULL, 
                       ranku = NULL, seed = NULL, cluster = TRUE, 
-                      ncores = NULL, ...) {
+                      ncores = NULL, ci.level = 0.95, ...) {
   
   if(kernel == "gaussian")  kernel <- 3
   if(kernel == "epanech")   kernel <- 1
@@ -154,9 +156,9 @@ localtest <- function(formula, data = data, na.action = "na.omit",
   if (missing(formula)) {
     stop("Argument \"formula\" is missing, with no default")
   }
-  if (missing(data)) {
-    stop("Argument \"data\" is missing, with no default")
-  }
+ # if (missing(data)) {
+#    stop("Argument \"data\" is missing, with no default")
+ # }
   
   if(!isTRUE(der %in% c(0, 1, 2))) {
     stop("",paste(der)," is not a r-th derivative implemented, only 
@@ -186,7 +188,9 @@ localtest <- function(formula, data = data, na.action = "na.omit",
   
   ncmax <- 5
   c2 <- NULL
-  # if(is.null(seed)) seed <- -1
+ 
+  
+  nalfas <- length(ci.level)
   
   
   
@@ -194,32 +198,68 @@ localtest <- function(formula, data = data, na.action = "na.omit",
   
   if (smooth != "splines") {
     
-    ffr <- interpret.frfastformula(formula, method = "frfast")
-    varnames <- ffr$II[2, ]
-    aux <- unlist(strsplit(varnames,split = ":"))
+    cl <- match.call()
+    mf <- match.call(expand.dots = FALSE)
+    m <- match(x = c("formula", "data", "subset", "weights", 
+                     "na.action", "offset"), table = names(mf), nomatch = 0L)
+    mf <- mf[c(1L, m)]
+    mf$drop.unused.levels <- TRUE
+    mf[[1L]] <- quote(stats::model.frame)
+    mf <- eval(expr = mf, envir = parent.frame())
+    mt <- attr(mf, "terms")
+    y <- model.response(mf, "numeric")
+    w <- as.vector(model.weights(mf))
+    if (!is.null(w) && !is.numeric(w)) 
+      stop("'weights' must be a numeric vector")
+    
+    terms <- attr(mt, "term.labels")
+    aux <- unlist(strsplit(terms,split = ":"))
     varnames <- aux[1]
+    namef <- aux[2]
+    response <- as.character(attr(mt, "variables")[2])
+    
     if (unlist(strsplit(varnames,split = ""))[1] == "s") {
       stop("Argument \"formula\" is wrong specified, see details of
            model specification in 'Details' of the frfast help." )
     }
-    namef <- aux[2]
-    if (length(aux) == 1) {f <- NULL}else{f <- data[ ,namef]}
-    newdata <- data
-    data <- data[ ,c(ffr$response, varnames)]
     
+    data <- mf
     
-    if (na.action == "na.omit"){ # ver la f
+    if (na.action == "na.omit"){ # ver la f, corregido
       data <- na.omit(data)
     }else{
       stop("The actual version of the package only supports 'na.omit' (observations are removed 
            if they contain any missing values)")
     }
-    #newdata <- na.omit(newdata[ ,varnames])
+    
+    if (length(aux) == 1) {f <- NULL}else{f <- data[ ,namef]}
     n <- nrow(data)
     
+    
+    
     }else{
+      
+      
       ffr <- interpret.gam(formula)
-      varnames <- ffr$pred.names[1]
+      cl <- match.call()
+      mf <- match.call(expand.dots = FALSE)
+      mf$formula <- ffr$fake.formula
+      
+      m <- match(x = c("formula", "data", "subset", "weights", 
+                       "na.action", "offset"), table = names(mf), nomatch = 0L)
+      mf <- mf[c(1L, m)]
+      mf$drop.unused.levels <- TRUE
+      mf[[1L]] <- quote(stats::model.frame)
+      mf <- eval(expr = mf, envir = parent.frame())
+      mt <- attr(mf, "terms")
+      y <- model.response(mf, "numeric")
+      w <- as.vector(model.weights(mf))
+      if (!is.null(w) && !is.numeric(w)) 
+        stop("'weights' must be a numeric vector")
+      
+      terms <- attr(mt, "term.labels")
+      response <- as.character(attr(mt, "variables")[2])
+      varnames <- terms[1]
       if (":" %in% unlist(strsplit(ffr$fake.names,split = ""))) {
         stop("Argument \"formula\" is wrong specified, see details of
              model specification in 'Details' of the frfast help." )
@@ -229,30 +269,112 @@ localtest <- function(formula, data = data, na.action = "na.omit",
                 model specification in 'Details' of the frfast help." )
       }
       
-      namef <- ffr$pred.names[2]
-      if (length(ffr$pred.names) == 1) {f <- NULL}else{f <- data[ ,namef]}
-      newdata <- data
-      
-      if (length(ffr$pred.names) == 1) {
-        data <- data[ ,c(ffr$response, varnames)]
-      }else{
-        data <- data[ ,c(ffr$response, varnames, namef)]
-      }
+      datam <- mf
       
       if (na.action == "na.omit"){
-        data <- na.omit(data)
+        datam <- na.omit(datam)
       }else{
-        stop("The actual version of the package only supports 'na.omit' (observations are removed 
-             if they contain any missing values)")
+        stop("The actual version of the package only supports 'na.omit' 
+             (observations are removed if they contain any missing values)")
       }
       
-      n <- nrow(data)
+      if (length(terms) == 1) {
+        f <- NULL
+        namef <- 1
+      }else{
+        namef <- terms[2]
+        f <- mf[ ,namef]
+      }
+      n <- nrow(datam)
       }
   
   
+  if(missing(data)) {
+    
+    response <- strsplit(response, "\\$")[[1]][2]
+    terms2 <- strsplit(terms, "\\$")
+    
+    if(length(terms) == 1){
+      formula <- as.formula(paste0(response, "~s(",terms2[[1]][2],")"))
+      names(datam) <- c(response, terms2[[1]][2])
+      varnames <- terms2[[1]][2]
+      namef <- "F"
+    }else{
+      formula <- as.formula(paste0(response, "~s(",terms2[[1]][2],", by = ", terms2[[2]][2],")"))
+      #data2 <- data
+      names(datam) <- c(response, terms2[[1]][2], terms2[[2]][2])
+      varnames <- terms2[[1]][2]
+      namef <- terms2[[2]][2]
+    }
+    
+    data <- datam
+  }
   
   
   
+  
+  # 
+  # if (smooth != "splines") {
+  #   
+  #   ffr <- interpret.frfastformula(formula, method = "frfast")
+  #   varnames <- ffr$II[2, ]
+  #   aux <- unlist(strsplit(varnames,split = ":"))
+  #   varnames <- aux[1]
+  #   if (unlist(strsplit(varnames,split = ""))[1] == "s") {
+  #     stop("Argument \"formula\" is wrong specified, see details of
+  #          model specification in 'Details' of the frfast help." )
+  #   }
+  #   namef <- aux[2]
+  #   if (length(aux) == 1) {f <- NULL}else{f <- data[ ,namef]}
+  #   newdata <- data
+  #   data <- data[ ,c(ffr$response, varnames)]
+  #   
+  #   
+  #   if (na.action == "na.omit"){ # ver la f
+  #     data <- na.omit(data)
+  #   }else{
+  #     stop("The actual version of the package only supports 'na.omit' (observations are removed 
+  #          if they contain any missing values)")
+  #   }
+  #   #newdata <- na.omit(newdata[ ,varnames])
+  #   n <- nrow(data)
+  #   
+  #   }else{
+  #     ffr <- interpret.gam(formula)
+  #     varnames <- ffr$pred.names[1]
+  #     if (":" %in% unlist(strsplit(ffr$fake.names,split = ""))) {
+  #       stop("Argument \"formula\" is wrong specified, see details of
+  #            model specification in 'Details' of the frfast help." )
+  #     }
+  #     if (length(ffr$smooth.spec) == 0) {
+  #       warning("Argument \"formula\" could be wrong specified without an 's', see details of
+  #               model specification in 'Details' of the frfast help." )
+  #     }
+  #     
+  #     namef <- ffr$pred.names[2]
+  #     if (length(ffr$pred.names) == 1) {f <- NULL}else{f <- data[ ,namef]}
+  #     newdata <- data
+  #     
+  #     if (length(ffr$pred.names) == 1) {
+  #       data <- data[ ,c(ffr$response, varnames)]
+  #     }else{
+  #       data <- data[ ,c(ffr$response, varnames, namef)]
+  #     }
+  #     
+  #     if (na.action == "na.omit"){
+  #       data <- na.omit(data)
+  #     }else{
+  #       stop("The actual version of the package only supports 'na.omit' (observations are removed 
+  #            if they contain any missing values)")
+  #     }
+  #     
+  #     n <- nrow(data)
+  #     }
+  # 
+  # 
+  # 
+  
+  
   if (is.null(f)) f <- rep(1, n)
   etiquetas <- unique(f)
   nf <- length(etiquetas)
@@ -304,8 +426,8 @@ localtest <- function(formula, data = data, na.action = "na.omit",
     
     localtest  <-.Fortran("localtest_",
                           f = as.integer(f),
-                          x = as.double(data[,varnames]),
-                          y = as.double(data[,ffr$response]),
+                          x = as.double(data[ ,varnames]),
+                          y = as.double(data[ ,response]),
                           w = as.double(weights),
                           n = as.integer(n),
                           h0 = as.double(h0),
@@ -323,22 +445,30 @@ localtest <- function(formula, data = data, na.action = "na.omit",
                           pcmin = as.double(rankl), # rango de busqueda maximo
                           r = as.integer(der),
                           D = as.double(rep(-1.0,1)),
-                          Ci = as.double(rep(-1.0,1)),
-                          Cs = as.double(rep(-1.0,1)),
+                          Ci = as.double(rep(-1.0,nalfas)),
+                          Cs = as.double(rep(-1.0,nalfas)),
                          # seed = as.integer(seed),
                           umatrix = as.double(umatrix),
+                         level = as.double(ci.level),
+                         nalfas = as.integer(nalfas),
                           PACKAGE = "npregfast"
     )
     
-    
-    
-    if (localtest$Ci <= 0 & 0 <= localtest$Cs) {
-      decision <- "Acepted"
-    } else {
-      decision <- "Rejected"
+    decision <- character(nalfas)
+    for(i in 1:nalfas){
+      if (localtest$Ci[i] <= 0 & 0 <= localtest$Cs[i]) {
+        decision[i] <- "Accepted"
+      } else {
+        decision[i] <- "Rejected"
+      }
     }
+    
+    
+    
+    
     res <- cbind(d = round(localtest$D, digits = 4), Lwr = round(localtest$Ci, digits = 4), 
-                 Upr = round(localtest$Cs, digits = 4), Decision = decision)
+                 Upr = round(localtest$Cs, digits = 4), Decision = decision,
+                 Ci.Level = round(ci.level, digits = 2))
     # class(res) <- 'localtest'
     
   }else{
@@ -348,7 +478,7 @@ localtest <- function(formula, data = data, na.action = "na.omit",
       # grid
       xgrid <- seq(min(data[ ,varnames]), max(data[ ,varnames]), length.out = kbin)
       newd <- expand.grid(xgrid, unique(f))
-      names(newd) <- ffr$pred.names
+      names(newd) <- c(varnames, namef)
       
       # estimations
       p <- array(NA, dim = c(kbin, 3, nf))
@@ -370,7 +500,7 @@ localtest <- function(formula, data = data, na.action = "na.omit",
                           FUN.VALUE = numeric(kfino))
       
       newdfino <- data.frame(as.vector(xgridfino), rep(unique(f), each = kfino))
-      names(newdfino) <- ffr$pred.names
+      names(newdfino) <- c(varnames, namef)
       
       # max
       muhatfino <- as.vector(predict(m, newdata = newdfino, type = "response"))
@@ -417,7 +547,7 @@ localtest <- function(formula, data = data, na.action = "na.omit",
     # bootstrap
     m <- gam(formula, weights = weights, data = data.frame(data, weights), ...)
     muhat <- as.vector(predict(m, type = "response"))
-    err <- data[, ffr$response] - muhat
+    err <- data[, response] - muhat
     err <- err - mean(err)
     yboot <- replicate(nboot, muhat + err *
                          sample(c(-sqrt(5) + 1, sqrt(5) + 1)/2, size = n,
@@ -427,22 +557,37 @@ localtest <- function(formula, data = data, na.action = "na.omit",
     i <- NULL
     d_allboot <- foreach(i = 1:nboot) %dopar% {
       datab <- data
-      datab[, ffr$response] <- yboot[, i]
+      datab[, response] <- yboot[, i]
       aux <- mainfun_localtest(formula, data = data.frame(datab, weights), 
                                weights = weights, ...)
       return(aux)
     }
     
     
-    ci <- quantile(unlist(d_allboot), probs = c(0.025, 0.975), na.rm = TRUE)
+    
+    
+    decision <- character(nalfas)
+    cilower <- numeric(nalfas)
+    ciupper <- numeric(nalfas)
+    
+    for(i in 1:nalfas){
+      alpha <- 1-ci.level[i]
+      ci <- quantile(unlist(d_allboot), 
+                     probs = c(alpha/2, 1 - (alpha/2)), na.rm = TRUE)
     
     if (ci[1] <= 0 & 0 <= ci[2]) {
-      decision <- "Acepted"
+      decision[i] <- "Accepted"
     } else {
-      decision <- "Rejected"
+      decision[i] <- "Rejected"
     }
-    res <- cbind(d = round(d, digits = 4), Lwr = round(ci[1], digits = 4), 
-                 Upr = round(ci[2], digits = 4), Decision = decision)
+    cilower[i] <- ci[1]
+    ciupper[i] <- ci[2] 
+    }
+    
+    
+    res <- cbind(d = round(d, digits = 4), Lwr = round(cilower, digits = 4), 
+                 Upr = round(ciupper, digits = 4), Decision = decision,
+                 Ci.Level = round(ci.level, digits = 2))
     
     rownames(res) <- NULL
     
